import json
import os
from myffe.tools.color_printer import ColorPrinter


def save_config(config, path='ffe_config.json'):
    """保存配置到文件
    
    Args:
        config (dict): 参数字典
        path (str): 保存路径，默认为 'ffe_config.json'
    
    Returns:
        bool: 保存是否成功
    """
    try:
        # 确保目录存在
        dir_path = os.path.dirname(path)
        if dir_path and not os.path.exists(dir_path):
            os.makedirs(dir_path)
        
        # 保存配置
        with open(path, 'w', encoding='utf-8') as f:
            json.dump(config, f, ensure_ascii=False, indent=2)
        
        ColorPrinter.print_success(f"配置已保存到 {path}")
        return True
    except Exception as e:
        ColorPrinter.print_error(f"保存配置失败: {e}")
        return False


def load_config(path='ffe_config.json'):
    """从文件加载配置
    
    Args:
        path (str): 配置文件路径，默认为 'ffe_config.json'
    
    Returns:
        dict: 加载的参数字典，如果加载失败返回空字典
    """
    try:
        if not os.path.exists(path):
            ColorPrinter.print_warning(f"配置文件 {path} 不存在")
            return {}
        
        with open(path, 'r', encoding='utf-8') as f:
            config = json.load(f)
        
        ColorPrinter.print_success(f"配置已从 {path} 加载")
        return config
    except Exception as e:
        ColorPrinter.print_error(f"加载配置失败: {e}")
        return {}


def get_default_config_path():
    """获取默认配置文件路径
    
    Returns:
        str: 默认配置文件路径
    """
    # 默认保存在当前目录
    return os.path.join(os.getcwd(), 'ffe_config.json')


def process_save_path(path):
    """
    处理保存路径，包括路径验证和交互式选择
    
    Args:
        path (str): 用户提供的路径
        
    Returns:
        str: 验证后的路径或None（如果跳过）
    """
    from pathlib import Path
    from datetime import datetime
    
    # 转换为Path对象
    path_obj = Path(path)
    
    # 检查路径是否存在
    if path_obj.exists():
        if path_obj.is_dir():
            # 如果是目录，使用默认文件名
            return str(path_obj / f"ffe_params_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json")
        else:
            # 如果是文件，直接使用
            return str(path_obj)
    
    # 路径不存在，搜索相似文件名
    similar_files = find_similar_files(path_obj)
    
    if similar_files:
        # 让用户从相似文件中选择
        return prompt_file_selection(similar_files)
    else:
        # 没有找到相似文件，询问用户
        return prompt_no_file_found(path)


def find_similar_files(path_obj):
    """
    在父目录中查找相似文件名
    
    Args:
        path_obj (Path): 目标路径
        
    Returns:
        list: 相似文件路径列表
    """
    similar_files = []
    target_name = path_obj.name
    
    # 在当前目录和父目录中搜索（最多2层）
    for i in range(3):  # 当前目录 + 2个父目录
        search_dir = path_obj.parent
        if not search_dir.exists():
            break
        
        # 查找名称中包含目标名称的文件
        for file in search_dir.iterdir():
            if file.is_file() and target_name in file.name:
                similar_files.append(str(file))
        
        # 向上移动一层
        path_obj = search_dir
    
    return similar_files


def prompt_file_selection(files):
    """
    提示用户从相似文件中选择
    
    Args:
        files (list): 相似文件路径列表
        
    Returns:
        str: 选中的文件路径或None（如果跳过）
    """
    try:
        from myffe.tools.interactive import interactive_select
        
        # 添加跳过选项
        options = files + ["skip"]
        
        ColorPrinter.print_info("找到相似文件:")
        for i, file in enumerate(options):
            ColorPrinter.print_info(f"{i+1}. {file}")
        
        # 使用交互式选择
        selected = interactive_select(
            "选择一个文件来保存参数，或跳过:",
            options,
            default=len(options)-1  # 默认选择跳过
        )
        
        if selected == "skip":
            ColorPrinter.print_info("跳过参数保存")
            return None
        else:
            return selected
    except ImportError:
        # 如果交互式模块不可用
        ColorPrinter.print_warning("交互式模块不可用，跳过参数保存")
        return None


def prompt_no_file_found(path):
    """
    当没有找到相似文件时提示用户
    
    Args:
        path (str): 原始路径
        
    Returns:
        str: 用户选择的路径或None（如果跳过）
    """
    try:
        from myffe.tools.interactive import interactive_select
        
        options = ["跳过", "使用原始路径（如果需要则创建）"]
        
        ColorPrinter.print_warning(f"文件不存在: {path}")
        ColorPrinter.print_info("未找到相似文件")
        
        selected = interactive_select(
            "您想要做什么?",
            options,
            default=0  # 默认选择跳过
        )
        
        if selected == "跳过":
            ColorPrinter.print_info("跳过参数保存")
            return None
        else:
            # 使用原始路径，必要时创建目录
            from pathlib import Path
            path_obj = Path(path)
            if not path_obj.parent.exists():
                path_obj.parent.mkdir(parents=True, exist_ok=True)
            return path
    except ImportError:
        # 如果交互式模块不可用
        ColorPrinter.print_warning("交互式模块不可用，跳过参数保存")
        return None



import numpy as np
from scipy import stats

def is_numeric_columns(col):
    """检查列是否为数值列"""
    for s in col:
        try:
            float(s)
        except ValueError:  
            return False
    return True

def detect_outliers_iqr(data, column):
    """使用IQR方法检测异常值"""
    Q1 = data[column].quantile(0.25)
    Q3 = data[column].quantile(0.75)
    IQR = Q3 - Q1
    lower_bound = Q1 - 1.5 * IQR
    upper_bound = Q3 + 1.5 * IQR
    outliers = data[(data[column] < lower_bound) | (data[column] > upper_bound)]
    return len(outliers) > 0

def detect_outliers_zscore(data, column):
    """使用Z-score方法检测异常值"""
    z_scores = np.abs(stats.zscore(data[column]))
    outliers = data[z_scores > 3]
    return len(outliers) > 0



import pandas as pd
import numpy as np
from scipy import stats
import time

# 导入所需模块
import sys
import os

# 添加MFM目录到路径
current_dir = os.path.dirname(os.path.abspath(__file__))
# py_tools -> route -> FFEv3_2 -> FFE -> MFM -> 根目录
root_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(current_dir)))))
if root_dir not in sys.path:
    sys.path.append(root_dir)

from myffe.tools.interactive import interactive
from myffe.tools.color_printer import ColorPrinter
from myffe.tools.len_match import len_match
from myffe.tools.visualize_route import visualize_route
from myffe.tools.interactive_utils import get_interactive_choice
from myffe.route.tools_py.scaler_selector import get_best_scaler

def analyze_standardization(data, auto_test=False):
    """分析数据的标准化需求，为每列推荐适合的标准化方法
    
    Args:
        data: pandas DataFrame，输入数据
        auto_test: bool，是否自动测试模式，默认为 False
    
    Returns:
        dict: 包含标准化参数的字典，使用嵌套字典列表结构
    """
    standard_params = {
        'standard_ways': []
    }

    scaler_methods = ["Z-score标准化", "Min-Max标准化", "MaxAbs标准化", "RobustScaler标准化", "QuantileTransformer标准化", "均值归一化", "单位长度归一化", "自定义缩放", "skip"]
    
    # 筛选出数值列
    numeric_columns = data.select_dtypes(include=['number']).columns.tolist()
    
    if not numeric_columns:
        ColorPrinter.print_info("数据中没有需要标准化的数值列")
        return {'standard_preprocessing': 'skip'}
    
    inter_standard = get_interactive_choice(
        title="智能推荐？",
        description="选择'ok'将自动为所有数值列推荐最佳标准化方法，选择'no'需要手动选择",
        auto_test=auto_test
    )
    
    if not inter_standard:
        while True:
            # 重置参数列表
            standard_params['standard_ways'] = []
            
            selected_cols = interactive(numeric_columns , title="请按上下箭头选择，回车确认标准化处理列,点击'm'键进行多选择", description="选择要进行标准化处理的列，可以多选", min_select=1, max_select=len(numeric_columns)).run()
            time.sleep(2)
            ColorPrinter.print_success('你已选择标准化处理列：' + str(selected_cols))
            
            # 为每列选择标准化方法
            for col in selected_cols:
                selected_method = interactive(scaler_methods, title=f"请按上下箭头选择，回车确认{col}的标准化方法", description="Z-score标准化:基于均值和标准差, Min-Max标准化:缩放到0-1范围, MaxAbs标准化:基于最大值, RobustScaler标准化:对异常值不敏感, QuantileTransformer标准化:将数据映射到均匀分布, 均值归一化:减去均值, 单位长度归一化:缩放到单位范数, 自定义缩放:根据数据特性自定义, skip:不进行标准化处理", min_select=1, max_select=1).run()
                # 确保返回值是单个字符串
                method = selected_method[0]
                standard_params['standard_ways'].append({'col': col, 'method': method})
                time.sleep(2)
                ColorPrinter.print_success(f'你已选择{col}的标准化方法：{method}')
            
            break
    else:
        # 智能推荐模式
        for col in numeric_columns:
            col_data = data[col].astype(str).tolist()
            best_method = get_best_scaler(col_data)
            standard_params['standard_ways'].append({'col': col, 'method': best_method})
            ColorPrinter.print_info(f"为列 '{col}' 推荐标准化方法：{best_method}")
    
    result = {
        'standard_preprocessing': standard_params
    }
    visualize_route(result)
    return result

def standard_py(data, auto_test=False):
    """Python备用方案
    
    Args:
        data: pandas DataFrame，输入数据
        auto_test: bool，是否自动测试模式，默认为 False
    
    Returns:
        dict: 包含标准化参数的字典
    """
    return analyze_standardization(data, auto_test=auto_test)

if __name__ == "__main__":
    # 测试代码
    import pandas as pd
    import numpy as np
    
    # 创建测试数据
    np.random.seed(42)
    data = pd.DataFrame({
        'numeric_col1': np.random.normal(100, 10, 20),
        'numeric_col2': np.random.normal(50, 5, 20),
        'numeric_col3': np.random.uniform(0, 100, 20)
    })
    
    ColorPrinter.print_info("测试数据:")
    ColorPrinter.print_any(data.head(), color='white')
    ColorPrinter.print_any("\n数据统计:", color='blue')
    ColorPrinter.print_any(data.describe(), color='white')
    
    # 测试标准化处理
    ColorPrinter.print_info("\n测试标准化处理:")
    result = standard_py(data)
    ColorPrinter.print_success("处理结果:")
    ColorPrinter.print_any(result, color='white')




"""
这个输入data数据
这里这个函数最终会在self.route里添加一个
下面格式的字典
'standard_preprocessing': {
    'standard_ways': [
        {'col': '列名', 'method': '标准化方法'},
        {'col': '列名', 'method': 'skip'}  # 跳过该列
    ]
}

method的取值有
    "Z-score标准化"   # Z-score标准化
    "Min-Max标准化"   # Min-Max标准化
    "MaxAbs标准化"   # MaxAbs标准化
    "RobustScaler标准化"   # RobustScaler标准化
    "QuantileTransformer标准化"   # QuantileTransformer标准化，将数据映射到均匀分布
    "均值归一化"   # 均值归一化，减去均值
    "单位长度归一化"   # 单位长度归一化，缩放到单位范数
    "自定义缩放"   # 自定义缩放，根据数据特性自定义
    "skip"   # 跳过标准化处理
"""




from setuptools import setup, find_packages

# Read README.md
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# Append update log
update_log_path = "myffe/update_log/1.1.1.md"
try:
    with open(update_log_path, "r", encoding="utf-8") as log_fh:
        update_log = log_fh.read()
    long_description += "\n\n" + update_log
except FileNotFoundError:
    pass

setup(
    name="myffe",        # this is package name
    version="1.1.1",     # this is the Version Number
    author="wbk",  # this is author's name 
    author_email="1757175380@qq.com",   #this is author's email    
    description="myffe - A powerful feature engineering framework for data preprocessing and machine learning",
    long_description=long_description,   #Import the library's README.md file as the long description
    long_description_content_type="text/markdown", #Specify the content type of the long description as Markdown
    url="https://github.com/nan-luoyan-nan/myffe", #this is the source code repository URL 
    project_urls={
        "Documentation": "https://github.com/nan-luoyan-nan/myffe",
        "Source": "https://github.com/nan-luoyan-nan/myffe",
        "Tracker": "https://github.com/nan-luoyan-nan/myffe/issues",
    },
    packages=find_packages(),     #from __init__ 
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],                                #These are the classifiers that describe the package's purpose and environment
    python_requires=">=3.7",
    install_requires=[      #this is the list of dependencies required to install the package
        "pandas>=1.3.0",
        "numpy>=1.20.0",
        "scikit-learn>=0.24.0",
        "scipy>=1.6.0",
        "imbalanced-learn>=0.8.0",
    ],
    keywords=[             #this is the list of keywords that describe the package's purpose and environment
        "feature engineering",
        "machine learning",
        "data preprocessing",
        "data cleaning",
        "data science",
        "python",
        "AI",
        "preprocessing",
        "feature extraction",
        "data analysis",
    ],
    include_package_data=True,  #include non-Python files specified in MANIFEST.in
    # MANIFEST.in is package list file
    package_data={
        "myffe": ["docs/*.json", "docs/*.txt", "learning/*.md", "update_log/*.md"],  #extra files to include in the package
    },
)



import sys
import os

# 添加项目根目录到Python路径
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..', '..', '..')))

import pandas as pd
import numpy as np
from myffe.tools.interactive import interactive
from myffe.tools.visualize_route import visualize_route
from myffe.tools.color_printer import ColorPrinter
from myffe.tools.param_utils import ensure_list, convert_to_list
from myffe.tools.interactive_utils import get_interactive_choice

def analyze_sample_distribution(data, label_columns):
    """分析数据的类别分布，推荐适合的采样处理方法"""
    # 初始化采样处理路线
    sample_route = {}
    
    # 对每个标签列进行分析
    for label_col in label_columns:
        # 检查该列是否存在于数据中
        if label_col not in data.columns:
            ColorPrinter.print_info(f"标签列 {label_col} 不存在于数据中，跳过采样处理")
            return {}
        
        # 检查该列是否适合作为标签列
        label_data = data[label_col]
        
        # 计算类别分布
        class_counts = label_data.value_counts()   # 计算每个类别的样本数量
        num_classes = len(class_counts)     # 类别数量
        total_samples = len(label_data)     # 总样本数量
        
        # 计算类别不平衡度
        if num_classes > 1:
            max_count = class_counts.max()
            min_count = class_counts.min()
            imbalance_ratio = max_count / min_count if min_count > 0 else float('inf')
        else:
            imbalance_ratio = 1.0
        
        # 分析数据特征，推荐适合的采样方法
        recommended_configs = []
        
        # 基于类别数量和不平衡度推荐方法
        if num_classes == 1:
            # 只有一个类别，不需要采样
            recommended_configs.append({
                'method': ['skip'],
                'balance_method': 'oversample',  # 占位
                'random_state': 42,  # 占位
                'high_sample_selection': 'RandomOverSampler',  # 占位
                'smote_k_neighbors': 5,  # 占位
                'adasyn_n_neighbors': 5,  # 占位
                'borderline_smote_k_neighbors': 5,  # 占位
                'svm_smote_k_neighbors': 5,  # 占位
                'enn_n_neighbors': 5,  # 占位
                'description': '跳过采样（只有一个类别）'
            })
        elif num_classes == 2:
            # 二分类问题
            if imbalance_ratio > 10:
                # 严重不平衡，推荐SMOTE或ADASYN
                recommended_configs.extend([
                    {
                        'method': ['MY_high_sample'],
                        'balance_method': 'oversample',  # 占位
                        'random_state': 42,  # 占位
                        'high_sample_selection': 'SMOTE',
                        'smote_k_neighbors': 5,
                        'adasyn_n_neighbors': 5,  # 占位
                        'borderline_smote_k_neighbors': 5,  # 占位
                        'svm_smote_k_neighbors': 5,  # 占位
                        'enn_n_neighbors': 5,  # 占位
                        'description': 'SMOTE采样（严重不平衡二分类）'
                    },
                    {
                        'method': ['MY_high_sample'],
                        'balance_method': 'oversample',  # 占位
                        'random_state': 42,  # 占位
                        'high_sample_selection': 'ADASYN',
                        'smote_k_neighbors': 5,  # 占位
                        'adasyn_n_neighbors': 5,
                        'borderline_smote_k_neighbors': 5,  # 占位
                        'svm_smote_k_neighbors': 5,  # 占位
                        'enn_n_neighbors': 5,  # 占位
                        'description': 'ADASYN采样（严重不平衡二分类）'
                    }
                ])
            elif imbalance_ratio > 2:
                # 中度不平衡，推荐RandomOverSampler或BorderlineSMOTE
                recommended_configs.extend([
                    {
                        'method': ['MY_high_sample'],
                        'balance_method': 'oversample',  # 占位
                        'random_state': 42,  # 占位
                        'high_sample_selection': 'RandomOverSampler',
                        'smote_k_neighbors': 5,  # 占位
                        'adasyn_n_neighbors': 5,  # 占位
                        'borderline_smote_k_neighbors': 5,  # 占位
                        'svm_smote_k_neighbors': 5,  # 占位
                        'enn_n_neighbors': 5,  # 占位
                        'description': 'RandomOverSampler采样（中度不平衡二分类）'
                    },
                    {
                        'method': ['MY_high_sample'],
                        'balance_method': 'oversample',  # 占位
                        'random_state': 42,  # 占位
                        'high_sample_selection': 'BorderlineSMOTE',
                        'smote_k_neighbors': 5,  # 占位
                        'adasyn_n_neighbors': 5,  # 占位
                        'borderline_smote_k_neighbors': 5,
                        'svm_smote_k_neighbors': 5,  # 占位
                        'enn_n_neighbors': 5,  # 占位
                        'description': 'BorderlineSMOTE采样（中度不平衡二分类）'
                    }
                ])
            else:
                # 平衡数据，推荐skip
                recommended_configs.append({
                    'method': ['skip'],
                    'balance_method': 'oversample',  # 占位
                    'random_state': 42,  # 占位
                    'high_sample_selection': 'RandomOverSampler',  # 占位
                    'smote_k_neighbors': 5,  # 占位
                    'adasyn_n_neighbors': 5,  # 占位
                    'borderline_smote_k_neighbors': 5,  # 占位
                    'svm_smote_k_neighbors': 5,  # 占位
                    'enn_n_neighbors': 5,  # 占位
                    'description': '跳过采样（平衡二分类）'
                })
        else:
            # 多分类问题
            if imbalance_ratio > 5:
                # 严重不平衡，推荐SMOTE或过采样
                recommended_configs.extend([     #extend方法将可迭代对象里面的东西添加到指定的列表的后面
                    {
                        'method': ['MY_high_sample'],
                        'balance_method': 'oversample',  # 占位
                        'random_state': 42,  # 占位
                        'high_sample_selection': 'SMOTE',
                        'smote_k_neighbors': 5,
                        'adasyn_n_neighbors': 5,  # 占位
                        'borderline_smote_k_neighbors': 5,  # 占位
                        'svm_smote_k_neighbors': 5,  # 占位
                        'enn_n_neighbors': 5,  # 占位
                        'description': 'SMOTE采样（严重不平衡多分类）'
                    },
                    {
                        'method': ['multi_class_balance'],
                        'balance_method': 'oversample',
                        'random_state': 42,
                        'high_sample_selection': 'RandomOverSampler',  # 占位
                        'smote_k_neighbors': 5,  # 占位
                        'adasyn_n_neighbors': 5,  # 占位
                        'borderline_smote_k_neighbors': 5,  # 占位
                        'svm_smote_k_neighbors': 5,  # 占位
                        'enn_n_neighbors': 5,  # 占位
                        'description': '过采样（严重不平衡多分类）'
                    }
                ])
            elif imbalance_ratio > 2:
                # 中度不平衡，推荐过采样
                recommended_configs.append({
                    'method': ['multi_class_balance'],
                    'balance_method': 'oversample',
                    'random_state': 42,
                    'high_sample_selection': 'RandomOverSampler',  # 占位
                    'smote_k_neighbors': 5,  # 占位
                    'adasyn_n_neighbors': 5,  # 占位
                    'borderline_smote_k_neighbors': 5,  # 占位
                    'svm_smote_k_neighbors': 5,  # 占位
                    'enn_n_neighbors': 5,  # 占位
                    'description': '过采样（中度不平衡多分类）'
                })
            else:
                # 平衡数据，推荐skip
                recommended_configs.append({
                    'method': ['skip'],
                    'balance_method': 'oversample',  # 占位
                    'random_state': 42,  # 占位
                    'high_sample_selection': 'RandomOverSampler',  # 占位
                    'smote_k_neighbors': 5,  # 占位
                    'adasyn_n_neighbors': 5,  # 占位
                    'borderline_smote_k_neighbors': 5,  # 占位
                    'svm_smote_k_neighbors': 5,  # 占位
                    'enn_n_neighbors': 5,  # 占位
                    'description': '跳过采样（平衡多分类）'
                })
        
        # 基于数据量大小调整推荐
        if total_samples < 1000:
            # 小数据集，优先推荐计算量较小的方法
            lightweight_configs = [config for config in recommended_configs if 
                                 ('multi_class_balance' in config.get('method', [])) or
                                 (config.get('high_sample_selection') in ['RandomOverSampler', 'RandomUnderSampler'])]
            if lightweight_configs:
                recommended_configs = lightweight_configs
        elif total_samples > 100000:
            # 大数据集，优先推荐高效的方法
            efficient_configs = [config for config in recommended_configs if 
                               ('multi_class_balance' in config.get('method', [])) or
                               (config.get('high_sample_selection') in ['RandomOverSampler', 'RandomUnderSampler'])]
            if efficient_configs:
                recommended_configs = efficient_configs
        
        # 为该标签列创建采样路线
        if recommended_configs:
            # 提取最佳配置
            best_config = recommended_configs[0]
            
            # 构建完整的参数配置，与参数配置文档和 sample_preprocessing.py 完全兼容
            # 新格式配置
            sample_params = {
                'sample_ways': [
                    {
                        'method': best_config.get('method', ['skip']),
                        'balance_method': best_config.get('balance_method', 'oversample'),
                        'random_state': best_config.get('random_state', 42),
                        'high_sample_selection': best_config.get('high_sample_selection', 'RandomOverSampler'),
                        'smote_k_neighbors': best_config.get('smote_k_neighbors', 5),
                        'adasyn_n_neighbors': best_config.get('adasyn_n_neighbors', 5),
                        'borderline_smote_k_neighbors': best_config.get('borderline_smote_k_neighbors', 5),
                        'svm_smote_k_neighbors': best_config.get('svm_smote_k_neighbors', 5),
                        'enn_n_neighbors': best_config.get('enn_n_neighbors', 5)
                    }
                ]
            }
            
            sample_route[label_col] = {
                'num_classes': num_classes,
                'total_samples': total_samples,
                'imbalance_ratio': round(imbalance_ratio, 2),
                'recommended_configs': recommended_configs,
                'best_config': best_config,
                'best_params': sample_params,
                'best_method': best_config['description']
            }
    
    return sample_route

def sample_py(data, label_columns, auto_test=False):
    """Python实现的采样处理分析"""
    # 确保label_columns是列表
    label_columns = ensure_list(label_columns)
    if not label_columns:
        ColorPrinter.print_info("没有提供标签列，跳过采样处理")
        return {'sample_preprocessing': 'skip'}
    
    #region 分析样本分布,分析错误直接跳过
    sample_route = analyze_sample_distribution(data, label_columns)
    
    if not sample_route:
        ColorPrinter.print_info("无法分析样本分布，跳过采样处理")
        return {'sample_preprocessing': 'skip'}
    #endregion
    
    # 获取第一个标签列的分析结果
    label_col = list(sample_route.keys())[0]
    analysis = sample_route[label_col]
    
#region 显示分析信息
    ColorPrinter.print_info(f"标签列: {label_col}")
    ColorPrinter.print_info(f"类别数量: {analysis['num_classes']}")
    ColorPrinter.print_info(f"总样本数量: {analysis['total_samples']}")
    ColorPrinter.print_info(f"类别不平衡度: {analysis['imbalance_ratio']}")
    ColorPrinter.print_info(f"推荐方法: {analysis['best_method']}")
#endregion
    #region 智能推荐菜单
    inter_choice = get_interactive_choice(
        title="采样处理",
        description="选择'ok'使用推荐方案，选择'no'自定义处理方案",
        auto_test=auto_test
    )
    #endregion
    
    if inter_choice:
        ColorPrinter.print_success("使用推荐方案")
        # 使用推荐参数
        sample_params = analysis['best_params']
        # 构建结果
        result = {'sample_preprocessing': sample_params}
        
        # 可视化处理路线
        visualize_route(result)
        return result
    else:
        # 自定义处理方案
        ColorPrinter.print_info("自定义处理方案")
        
#region 初始化参数
        sample_params = {
            'sample_ways': [
                {
                    'method': ['skip'],
                    'balance_method': 'oversample',
                    'random_state': 42,
                    'high_sample_selection': 'RandomOverSampler',
                    'smote_k_neighbors': 5,
                    'adasyn_n_neighbors': 5,
                    'borderline_smote_k_neighbors': 5,
                    'svm_smote_k_neighbors': 5,
                    'enn_n_neighbors': 5
                }
            ]
        }
#endregion
        
        # 选择处理方法
        method_options = ["multi_class_balance", "MY_high_sample", "skip"]
        method_selected = interactive(method_options, title="选择处理方法", description="请选择采样方法:", min_select=1, max_select=1).run()
        selected_method = method_selected[0] if method_selected else "skip"
        
        # 更新新格式
        sample_params['sample_ways'][0]['method'] = [selected_method]
        
        if selected_method == "multi_class_balance":
            # 选择平衡方法
            balance_options = ["oversample", "undersample"]
            balance_selected = interactive(balance_options, title="选择平衡方法", description="请选择平衡方法:", min_select=1, max_select=1).run()
            balance_method = balance_selected[0] if balance_selected else "oversample"
            
            # 更新新格式
            sample_params['sample_ways'][0]['balance_method'] = balance_method
            
            # 输入随机种子
            random_state = input("请输入随机种子 (默认42): ")
            random_state_val = int(random_state) if random_state else 42
            
            # 更新新格式
            sample_params['sample_ways'][0]['random_state'] = random_state_val
        elif selected_method == "MY_high_sample":
            # 选择高级采样方法
            high_sample_options = [
                "SMOTE",
                "ADASYN",
                "BorderlineSMOTE",
                "SVMSMOTE",
                "RandomOverSampler",
                "RandomUnderSampler",
                "TomekLinks",
                "EditedNearestNeighbours",
                "ClusterCentroids"
            ]
            high_sample_selected = interactive(high_sample_options, title="选择高级采样方法", description="请选择高级采样方法:", min_select=1, max_select=1).run()
            high_sample_selection = high_sample_selected[0] if high_sample_selected else "RandomOverSampler"
            
            # 更新新格式
            sample_params['sample_ways'][0]['high_sample_selection'] = high_sample_selection
            
            # 根据选择的方法添加相应参数
            if high_sample_selection == "SMOTE":
                smote_k = input("请输入SMOTE邻居数 (默认5): ")
                smote_k_val = int(smote_k) if smote_k else 5
                # 更新新格式
                sample_params['sample_ways'][0]['smote_k_neighbors'] = smote_k_val
            elif high_sample_selection == "ADASYN":
                adasyn_k = input("请输入ADASYN邻居数 (默认5): ")
                adasyn_k_val = int(adasyn_k) if adasyn_k else 5
                # 更新新格式
                sample_params['sample_ways'][0]['adasyn_n_neighbors'] = adasyn_k_val
            elif high_sample_selection == "BorderlineSMOTE":
                borderline_k = input("请输入BorderlineSMOTE邻居数 (默认5): ")
                borderline_k_val = int(borderline_k) if borderline_k else 5
                # 更新新格式
                sample_params['sample_ways'][0]['borderline_smote_k_neighbors'] = borderline_k_val
            elif high_sample_selection == "SVMSMOTE":
                svm_k = input("请输入SVMSMOTE邻居数 (默认5): ")
                svm_k_val = int(svm_k) if svm_k else 5
                # 更新新格式
                sample_params['sample_ways'][0]['svm_smote_k_neighbors'] = svm_k_val
            elif high_sample_selection == "EditedNearestNeighbours":
                enn_k = input("请输入EditedNearestNeighbours邻居数 (默认5): ")
                enn_k_val = int(enn_k) if enn_k else 5
                # 更新新格式
                sample_params['sample_ways'][0]['enn_n_neighbors'] = enn_k_val
    
    # 构建最终结果
    result = {'sample_preprocessing': sample_params}
        
    # 可视化处理路线
    visualize_route(result)
        
    return result


if __name__ == "__main__":
    """
    测试采样处理功能
    """
    import pandas as pd
    import numpy as np
    
    print("=" * 80)
    print("采样处理模块测试")
    print("=" * 80)
    
    # 创建测试数据
    np.random.seed(42)
    data = pd.DataFrame({
        'feature1': np.random.normal(50, 10, 100),
        'feature2': np.random.normal(100, 20, 100),
        'feature3': np.random.normal(200, 30, 100),
        'label': ['A'] * 70 + ['B'] * 20 + ['C'] * 10
    })
    
    print("测试数据:")
    print(data)
    print(f"\n数据形状: {data.shape}")
    print(f"列名: {list(data.columns)}")
    print(f"类别分布:")
    print(data['label'].value_counts())
    
    print("\n" + "=" * 80)
    print("测试标签列检测")
    print("=" * 80)
    
    # 测试标签列检测
    label_cols = ['label']  # 直接使用已知的标签列
    print(f"使用的标签列: {label_cols}")
    
    print("\n" + "=" * 80)
    print("测试样本分布分析")
    print("=" * 80)
    
    # 测试样本分布分析
    sample_route = analyze_sample_distribution(data, label_cols)
    if sample_route:
        label_col = list(sample_route.keys())[0]
        analysis = sample_route[label_col]
        print(f"标签列: {label_col}")
        print(f"类别数量: {analysis['num_classes']}")
        print(f"总样本数量: {analysis['total_samples']}")
        print(f"类别不平衡度: {analysis['imbalance_ratio']}")
        print(f"推荐方法: {analysis['best_method']}")
        print(f"推荐参数: {analysis['best_params']}")
    
    print("\n" + "=" * 80)
    print("测试主处理函数")
    print("=" * 80)
    
    # 测试主处理函数
    try:
        result = sample_py(data, label_columns=['label'])
        print("\n处理结果:")
        if result == 'skip':
            print("跳过处理")
        else:
            for key, value in result.items():
                print(f"\n{key}:")
                for param_key, param_value in value.items():
                    print(f"  {param_key}: {param_value}")
    except Exception as e:
        print(f"测试过程中出错: {e}")
    
    print("\n" + "=" * 80)
    print("测试完成!")
    print("=" * 80)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
路线管理器

整合了 route_config.py 和 auto_route.py 的功能，提供统一的路线配置和管理接口。
"""

import os
import sys
import pandas as pd
import json

from myffe.tools.color_printer import ColorPrinter

class RouteManager:
    """路线管理器类
    
    整合了 route_config.py 和 auto_route.py 的功能，提供统一的路线配置和管理接口。
    支持手动配置和自动推荐两种模式。
    """
    
    def __init__(self, tools=None, label_columns=None):
        """初始化路线管理器
        
        Args:
            tools (list, optional): 要配置的工具列表，默认None（执行所有工具）
            label_columns (list, optional): 标签列，默认None
        """
        self.tools = tools
        self.label_columns = label_columns
        self.route = {}
    
    def __call__(self, data, label_columns=None):
        """主调用函数
        
        Args:
            data (DataFrame): 输入数据，pandas DataFrame格式
            label_columns (list, optional): 标签列，默认N  one
        
        Returns:
            dict: 推荐的特征工程路线
        """
        # 如果没有提供label_columns，使用初始化时的参数
        if label_columns is None:
            label_columns = self.label_columns
        
        self.data = data
        self.route = self.get_route(label_columns)
        return self.route
    
    def get_route(self, label_columns=None):
        """获取路线配置
        
        Args:
            label_columns (list, optional): 标签列，默认None
        
        Returns:
            dict: 特征工程路线配置
        """
        # 初始化路线结构
        self.route = {}
        
        # 定义工具配置，包含路由名称、工具函数名称和参数
        tool_configs = {
            'nan_preprocessing': ('nan', 'nan_py', ()),
            'sample_preprocessing': ('sample', 'sample_py', (label_columns,)),
            'outlier_preprocessing': ('outlier', 'outlier_py', ()),
            'str_preprocessing': ('str', 'str_py', ()),
            'relative_preprocessing': ('relative', 'relative_py', ()),
            'time_preprocessing': ('time', 'time_py', ()),
            'specify_preprocessing': ('specify', 'specify_py', ()),
            'standard_preprocessing': ('standard', 'standard_py', ()),
            'normalized_preprocessing': ('normalized', 'normalized_py', ())
        }
        
        # 按照顺序执行工具
        # 如果没有指定工具列表，则执行所有工具
        if self.tools is None:
            # 按顺序执行所有工具
            tools_order = [
                'nan_preprocessing',
                'sample_preprocessing',
                'outlier_preprocessing',
                'str_preprocessing',
                'relative_preprocessing',
                'time_preprocessing',
                'specify_preprocessing',
                'standard_preprocessing',
                'normalized_preprocessing'
            ]
            for tool in tools_order:
                route_name, tool_name, args = tool_configs[tool]
                # 对于sample_preprocessing，需要传递原始数据（包含标签列）
                if tool == 'sample_preprocessing':
                    self._process_route(route_name, tool_name, *args, use_original_data=True, label_columns=label_columns)
                else:
                    self._process_route(route_name, tool_name, *args, label_columns=label_columns)
        else:
            # 只执行指定的工具
            for tool in self.tools:
                if tool in tool_configs:
                    route_name, tool_name, args = tool_configs[tool]
                    # 对于sample_preprocessing，需要传递原始数据（包含标签列）
                    if tool == 'sample_preprocessing':
                        self._process_route(route_name, tool_name, *args, use_original_data=True, label_columns=label_columns)
                    else:
                        self._process_route(route_name, tool_name, *args, label_columns=label_columns)
        
        # 可视化路由数据
        try:
            # 导入visualize_route模块（从tools目录）
            from myffe.tools import visualize_route
            ColorPrinter.print_info("\n" + "=" * 80)
            ColorPrinter.print_info("参数配置可视化")
            ColorPrinter.print_info("=" * 80)
            visualize_route(self.route)
        except ImportError as e:
            ColorPrinter.print_error(f"\n可视化模块未找到: {e}")
            ColorPrinter.print_error("请确保 visualize_route.py 在 tools 目录中")
        
        return self.route
    
    def _process_route(self, route_name, tool_name, *args, use_original_data=False, label_columns=None, **kwargs):
        """通用处理方法，简化重复的路由处理逻辑
        
        Args:
            route_name (str): 路由名称
            tool_name (str): 工具函数名称
            *args: 传递给工具函数的参数
            use_original_data (bool): 是否使用原始数据（包含标签列）
            label_columns (list): 标签列
            **kwargs: 传递给工具函数的关键字参数
        """
        # 动态导入工具模块
        module = __import__(f'myffe.route.py_tools.{tool_name}', fromlist=[tool_name])
        tool_func = getattr(module, tool_name)
        
        # 选择要传递的数据
        if use_original_data:
            # 传递原始数据（包含标签列）
            data_to_pass = self.data
        else:
            # 分离标签列和特征数据
            if label_columns:
                # 确保label_columns是列表
                if isinstance(label_columns, str):
                    label_columns = [label_columns]
                # 分离特征数据（排除标签列）
                data_to_pass = self.data[[col for col in self.data.columns if col not in label_columns]]
            else:
                data_to_pass = self.data.copy()
        
        # 调用工具函数获取处理参数，传递auto_test=True
        result = tool_func(data_to_pass, *args, auto_test=True, **kwargs)
        
        # 处理返回结果，确保结构正确
        if isinstance(result, dict):
            # 检查是否已经包含_preprocessing键
            preprocessing_key = f'{route_name}_preprocessing'
            if preprocessing_key in result:
                # 如果已经包含，则直接使用内部的内容
                self.route[preprocessing_key] = result[preprocessing_key]
            else:
                # 否则直接使用结果
                self.route[preprocessing_key] = result
        elif result == 'skip':
            # 如果返回的是'skip'字符串，创建包含'skip'值的默认参数字典
            preprocessing_key = f'{route_name}_preprocessing'
            # 根据工具类型创建对应的默认参数字典
            if route_name == 'nan':
                self.route[preprocessing_key] = {'nan_missing_process_choice': ['skip'], 'nan_processing_col': []}
            elif route_name == 'sample':
                self.route[preprocessing_key] = {'choice1_normal': ['skip'], 'choice1_advance': ['skip']}
            elif route_name == 'time':
                self.route[preprocessing_key] = {'time_col': None}
            elif route_name == 'str':
                self.route[preprocessing_key] = {'str_choice': 'skip', 'str_cols': []}
            elif route_name == 'outlier':
                self.route[preprocessing_key] = {'outlier_choice': 'skip'}
            elif route_name == 'relative':
                self.route[preprocessing_key] = {'choice3_normal': ['skip'], 'choice3_advance': ['skip']}
            elif route_name == 'specify':
                self.route[preprocessing_key] = {'drop_col': [], 'drop_row': []}
            elif route_name == 'standard':
                self.route[preprocessing_key] = {'scaler_method_choice': [], 'standard_col': []}
            elif route_name == 'normalized':
                self.route[preprocessing_key] = {'normalizer_method_choice': [], 'normalized_col': []}
            else:
                # 其他工具类型的默认处理
                self.route[preprocessing_key] = {'choice': 'skip'}
        else:
            # 如果返回的不是字典也不是'skip'，直接存储
            self.route[f'{route_name}_preprocessing'] = result
    
    def read(self, path):
        """读取数据集
        
        Args:
            path (str): 数据集路径
        
        Returns:
            DataFrame: 读取的数据集
        """
        # 统一使用pandas读取文件
        ColorPrinter.print_info("使用pandas读取CSV文件")
        return pd.read_csv(path)
    
    def get_cleaning_info(self):
        """获取清理信息（只返回路线信息，不执行清理）
        
        Returns:
            str: 格式化的路线信息JSON字符串
        """
        # 返回格式化的JSON字符串，便于阅读
        return json.dumps(self.route, indent=2, ensure_ascii=False)


# 使用示例
if __name__ == "__main__":
    # 创建测试数据
    import csv
    
    test_data = [
        ["name", "age", "city", "salary", "score", "label"],
        ["Alice", "25", "New York", "50000", "85", "A"],
        ["Bob", "", "Los Angeles", "60000", "90", "B"],  # 空值
        ["", "30", "Chicago", "70000", "75", "A"],        # 空值
        ["David", "35", "", "80000", "80", "C"],          # 空值
        ["Eve", "40", "Boston", "90000", "95", "A"]
    ]
    
    # 写入测试文件
    with open('test.csv', 'w', newline='') as f:
        writer = csv.writer(f)
        writer.writerows(test_data)
    
    # 创建RouteManager实例
    route_manager = RouteManager()
    
    # 读取数据
    data = route_manager.read('test.csv')
    
    # 获取路线配置
    route = route_manager(data, label_columns=['label'])
    
    ColorPrinter.print_info("清理路线:")
    if 'specify_preprocessing' in route:
        if 'drop_row' in route['specify_preprocessing']:
            ColorPrinter.print_info(f"要删除的行: {route['specify_preprocessing']['drop_row']}")
        if 'drop_col' in route['specify_preprocessing']:
            ColorPrinter.print_info(f"要删除的列: {route['specify_preprocessing']['drop_col']}")
    if 'nan_preprocessing' in route:
        ColorPrinter.print_info(f"缺失值处理方法: {route['nan_preprocessing'].get('nan_missing_process_choice', 'None')}")
    
    if 'standard_preprocessing' in route:
        ColorPrinter.print_info("\n标准化路线:")
        standard_col = route['standard_preprocessing']['standard_col']
        ColorPrinter.print_info(f"标准化列: {standard_col}")
        ColorPrinter.print_info("标准化方法:")
        methods = route['standard_preprocessing']['scaler_method_choice']
        for col, method in zip(standard_col, methods):
            ColorPrinter.print_info(f"  {col}: {method}")
    
    if 'normalized_preprocessing' in route:
        ColorPrinter.print_info("\n归一化路线:")
        normalized_col = route['normalized_preprocessing']['normalized_col']
        ColorPrinter.print_info(f"归一化列: {normalized_col}")
        ColorPrinter.print_info("归一化方法:")
        methods = route['normalized_preprocessing']['normalizer_method_choice']
        for col, method in zip(normalized_col, methods):
            ColorPrinter.print_info(f"  {col}: {method}")
    
    if 'sample_preprocessing' in route:
        ColorPrinter.print_info("\n采样处理路线:")
        sample_info = route['sample_preprocessing']
        ColorPrinter.print_info("采样参数配置:")
        for key, value in sample_info.items():
            ColorPrinter.print_info(f"    {key}: {value}")
    
    # 获取清理信息（不执行清理）
    cleaning_info = route_manager.get_cleaning_info()
    ColorPrinter.print_info("\n清理信息:")
    ColorPrinter.print_info(cleaning_info)



import os
import sys

# the '@classmethod' can be used to define a method that belongs to the class rather than an instance of the class.
# In this case, the method can be called without creating an instance of the class.
# such as ColorPrinter.print_success('Success!')

class ColorPrinter:
    """Color printing utility class"""

    # Check if ANSI colors are supported
    @classmethod
    def _supports_color(cls):
        # Check if color is explicitly disabled
        if os.environ.get('NO_COLOR'):
            return False
        # Check if the terminal supports color
        if os.name == 'nt':
            # Windows: Always try to enable color for modern terminals
            # Try to enable ANSI virtual terminal processing
            try:
                import ctypes
                kernel32 = ctypes.windll.kernel32
                handle = kernel32.GetStdHandle(-11)
                mode = ctypes.c_ulong()
                kernel32.GetConsoleMode(handle, ctypes.byref(mode))
                ENABLE_VIRTUAL_TERMINAL_PROCESSING = 0x0004
                kernel32.SetConsoleMode(handle, mode.value | ENABLE_VIRTUAL_TERMINAL_PROCESSING)
            except:
                pass
            # Check for common color-supporting terminal environments
            if os.environ.get('WT_SESSION') or os.environ.get('TERM_PROGRAM'):
                return True
            # Also enable for known terminal emulators
            term = os.environ.get('TERM', '')
            if 'color' in term.lower() or term.startswith('xterm') or term.startswith('screen'):
                return True
            return False
        return hasattr(sys.stdout, 'isatty') and sys.stdout.isatty()

    COLORS = {     # \033 or \x1b is the escape character for ANSI color codes.
                   # the 0m is the reset code to restore the default color.
                   # the '[' is the start of the color code 
        'red': '\033[91m',
        'green': '\033[92m',
        'yellow': '\033[93m',
        'blue': '\033[94m',
        'magenta': '\033[95m',
        'cyan': '\033[96m',
        'white': '\033[97m',
        'reset': '\033[0m'
    }

    @classmethod    # while used the @classmethod ,the cls is the class itself rather than an instance of the class.
    def print(cls, message, color='white'):
        if cls._supports_color():
            color_code = cls.COLORS.get(color, cls.COLORS['white'])
            reset_code = cls.COLORS['reset']
            console_message = f"{color_code}{message}{reset_code}"
            print(console_message)
        else:
            # In terminals that don't support color, print message directly
            print(message)

    @classmethod
    def print_any(cls, *objects, sep=' ', end='\n', color=None):
        text = sep.join(str(obj) for obj in objects)
        if color and cls._supports_color():
            color_code = cls.COLORS.get(color, cls.COLORS['white'])
            reset_code = cls.COLORS['reset']
            console_message = f"{color_code}{text}{reset_code}"
            print(console_message, end=end)
        else:
            print(text, end=end)

    @classmethod
    def print_success(cls, message):
        cls.print(message, 'green')

    @classmethod
    def print_error(cls, message):
        cls.print(message, 'red')

    @classmethod
    def print_warning(cls, message):
        cls.print(message, 'yellow')

    @classmethod
    def print_info(cls, *objects, sep=' '):
        message = sep.join(str(obj) for obj in objects)
        cls.print(message, 'blue')

    @classmethod
    def print_data(cls, message):
        cls.print(message, 'magenta')

    @classmethod
    def print_progress(cls, message):
        cls.print(message, 'cyan')



import numpy as np

def is_numeric_columns(col):
    """判断列是否为数值类型"""
    for s in col:
        try:
            float(s)
        except ValueError:
            return False
    return True

def select_best_normalizer(x):
    """选择最佳归一化方法"""
    n = len(x)
    
    # 计算各种归一化方法的评分
    score_l1 = 0.0
    score_l2 = 0.0
    score_max = 0.0
    score_robust = 0.0
    score_power = 0.0
    score_log = 0.0
    score_sqrt = 0.0
    
    # 计算基本统计量
    mean = np.mean(x)
    variance = np.var(x)
    std_dev = np.sqrt(variance)
    max_val = max(x)
    min_val = min(x)
    range_val = max_val - min_val
    q1 = np.percentile(x, 25)
    q3 = np.percentile(x, 75)
    iqr = q3 - q1
    
    # 计算L1评分 - 基于数据的分布特性，而不是仅仅稀疏性
    if range_val > 0:
        # 对于非稀疏数据，基于数据的均匀性评分
        # 计算数据的变异系数
        cv = std_dev / mean if mean > 0 else 0
        # 变异系数适中时L1评分较高
        score_l1 = np.exp(-0.5 * ((cv - 1.0) ** 2))
    else:
        score_l1 = 0.0
    
    # 计算L2评分 - 基于数据的分布均匀性
    if std_dev > 0:
        # 标准差适中时得分最高
        # 调整参数，使标准差在0.5-2之间时得分较高
        score_l2 = np.exp(-0.5 * ((np.log(std_dev) - 0) ** 2))
    else:
        score_l2 = 0.0
    
    # 计算Max评分 - 基于数据的范围
    if range_val > 0:
        # 对于不同范围的数据，都有合理的评分
        # 调整参数，使范围在1-1000之间时都能得到合理的评分
        score_max = 1.0 / (1.0 + np.exp(-0.01 * (np.log10(range_val + 1) - 1)))
    else:
        score_max = 0.0
    
    # 计算Robust评分 - 基于数据的异常值情况
    if iqr > 0 and range_val > 0:
        # IQR与范围的比例越大，说明异常值越多，越适合Robust归一化
        outlier_ratio = iqr / range_val
        score_robust = outlier_ratio
    else:
        score_robust = 0.0
    
    # 计算幂次归一化评分 - 基于数据的偏度
    if std_dev > 0:
        skewness = np.abs(np.mean(((x - mean) / std_dev) ** 3))
        # 偏度越大，越适合使用幂次归一化
        score_power = min(skewness / 2, 1.0)  # 偏度为2时得分约为1.0
    else:
        score_power = 0.0
    
    # 计算对数归一化评分 - 基于数据的取值范围和偏度
    if all(v > 0 for v in x):  # 对数归一化要求所有值为正
        # 数据范围较大且偏度较大时适合对数归一化
        if max_val > 1:
            log_range = np.log10(max_val) - np.log10(min_val)
            # 计算数据的偏度
            if std_dev > 0:
                skewness = np.mean(((x - mean) / std_dev) ** 3)
                # 正偏度较大时更适合对数归一化
                if skewness > 0:
                    score_log = min((log_range / 3 + skewness / 3), 1.0)
                else:
                    score_log = min(log_range / 5, 1.0)
            else:
                score_log = min(log_range / 5, 1.0)
        else:
            score_log = 0.0
    else:
        score_log = 0.0
    
    # 计算平方根归一化评分 - 基于数据的取值范围和分布
    if all(v >= 0 for v in x):  # 平方根归一化要求所有值非负
        # 数据为非负数且有一定范围时适合平方根归一化
        if max_val > 0:
            # 计算数据的偏度
            if std_dev > 0:
                skewness = np.mean(((x - mean) / std_dev) ** 3)
                # 正偏度适中时更适合平方根归一化
                if skewness > 0 and skewness < 2:
                    score_sqrt = min((np.sqrt(range_val) / 10 + skewness / 2), 1.0)
                else:
                    score_sqrt = min(np.sqrt(range_val) / 15, 1.0)
            else:
                score_sqrt = min(np.sqrt(range_val) / 15, 1.0)
        else:
            score_sqrt = 0.0
    else:
        score_sqrt = 0.0
    
    # 为不同类型的列添加特定的偏好
    # 检测列的类型（基于值的范围和分布）
    is_id_column = max_val > 100 and min_val == 1 and np.allclose(np.diff(sorted(x)), 1)  # 连续ID
    is_age_column = min_val >= 0 and max_val <= 120 and range_val > 5  # 年龄列
    is_score_column = min_val >= 0 and max_val <= 10 and range_val < 5  # 分数列（如GPA）
    
    # 根据列类型调整评分
    if is_id_column:
        # ID列通常不需要归一化，但如果必须选择，推荐Max或L2
        score_max *= 1.5
        score_l2 *= 1.2
    elif is_age_column:
        # 年龄列适合L2或Max归一化
        score_l2 *= 1.3
        score_max *= 1.2
    elif is_score_column:
        # 分数列适合L2或Robust归一化
        score_l2 *= 1.3
        score_robust *= 1.2
    
    # 最后进行比较，返回评分最高的那个归一化方法
    scores = [score_l1, score_l2, score_max, score_robust, score_power, score_log, score_sqrt]
    best_index = np.argmax(scores)
    
    # 返回对应的归一化方法
    normalizer_methods = ['L1 Normalization', 'L2 Normalization', 'Max Normalization', 'Robust Normalization', 'Power Normalization', 'Log Normalization', 'Square Root Normalization']
    return normalizer_methods[best_index]

def get_best_normalizer(col):
    """获取最佳归一化方法"""
    data = str_to_double(col)
    return select_best_normalizer(data)

def str_to_double(col):
    """将字符串列转换为double数组"""
    return [float(s) for s in col]



import pandas as pd
import numpy as np
import asyncio
from myffe.tools.color_printer import ColorPrinter

async def detect_time_columns(data):
    """异步检测时间列
    
    Args:
        data: pandas DataFrame，输入数据
        
    Returns:
        list: 检测到的时间列列表
    """
    time_columns = []
    # 常见的时间列名称
    time_column_names = [
        'date', 'time', 'timestamp', 'datetime', 'Date', 'Time', 'Timestamp', 'Datetime',
        'created_at', 'updated_at', 'start_time', 'end_time', 'event_time', 'record_time',
        'order_time', 'purchase_time', 'transaction_time', 'register_time', 'login_time',
        'created', 'updated', 'start', 'end', 'event', 'record',
        'order', 'purchase', 'transaction', 'register', 'login'
    ]
    
    # 首先检查常见的时间列名称
    for col in data.columns:
        if col.lower() in [name.lower() for name in time_column_names]:
            time_columns.append(col)
    
    # 检查已经是datetime类型的列
    for col in data.columns:
        if pd.api.types.is_datetime64_any_dtype(data[col]):
            if col not in time_columns:
                time_columns.append(col)
    
    # 尝试检测具有时间格式的列
    if not time_columns:
        for col in data.columns:
            # 跳过数字列
            if pd.api.types.is_numeric_dtype(data[col]):
                continue
                
            # 处理字符串类型的列
            if data[col].dtype == 'object' or data[col].dtype == 'string' or str(data[col].dtype) == 'str':
                try:
                    # 尝试将列转换为数字，如果能转换则排除
                    numeric_data = pd.to_numeric(data[col], errors='coerce')
                    if numeric_data.notnull().mean() > 0.9:  # 90%以上能转换为数字，则排除
                        continue
                        
                    # 只取第一行数据进行测试
                    first_value = data[col].iloc[0]
                    if pd.isna(first_value):
                        continue
                        
                    # 检查第一行是否为时间格式
                    time_data = pd.to_datetime(first_value, errors='coerce')
                    if pd.notna(time_data):
                        # 检查整体转换率
                        full_time_data = pd.to_datetime(data[col], errors='coerce')
                        conversion_rate = full_time_data.notnull().mean()
                        if conversion_rate > 0.6:  # 至少60%的数据可以转换为时间
                            time_columns.append(col)
                except Exception as e:
                    ColorPrinter.print_warning(f"检测列 {col} 时出错: {e}")
    
    return time_columns

async def analyze_time_column(data, col):
    """异步分析时间列特征
    
    Args:
        data: pandas DataFrame，输入数据
        col: 时间列名
        
    Returns:
        dict: 时间列特征分析结果
    """
    result = {
        'column': col,
        'has_time_info': False,
        'recommended_type': 'basic'
    }
    
    try:
        # 检查时间列是否包含时分秒信息
        test_data = pd.to_datetime(data[col], errors='coerce')
        
        # 检查是否有小时信息
        has_time_info = False
        try:
            hour_info = test_data.dt.hour
            has_time_info = not hour_info.isnull().all()
            result['has_time_info'] = has_time_info
        except Exception:
            pass
        
        # 检查数据量，数据量较大时推荐高级处理
        if len(data) > 1000 or has_time_info:
            result['recommended_type'] = 'advance'
    except Exception as e:
        ColorPrinter.print_warning(f"分析列 {col} 时出错: {e}")
    
    return result

async def batch_analyze_time_columns(data, time_columns):
    """批量分析时间列
    
    Args:
        data: pandas DataFrame，输入数据
        time_columns: 时间列列表
        
    Returns:
        list: 时间列分析结果列表
    """
    tasks = []
    for col in time_columns:
        task = asyncio.create_task(analyze_time_column(data, col))
        tasks.append(task)
    
    results = await asyncio.gather(*tasks)
    return results



from myffe.tools.interactive import interactive
from myffe.tools.color_printer import ColorPrinter

def get_interactive_choice(title, description, auto_test=False):
    """获取用户的交互式选择
    
    Args:
        title: 交互标题
        description: 交互描述
        auto_test: 是否自动测试模式
    
    Returns:
        bool: 是否使用智能推荐
    """
    inter_choice = False
    
    if auto_test:
        print("\033[31m下面进行测试使用智能推荐\033[0m")
        inter_choice = True
    else:
        inter_tem = interactive(['ok', 'no'], title=title, description=description, min_select=1, max_select=1).run()
        if not inter_tem:
            inter_choice = False
        else:
            inter_tem = inter_tem[0]
            if inter_tem == 'ok':
                inter_choice = True
            else:
                inter_choice = False
    
    return inter_choice



"""
NanPreprocessing module for handling missing values in datasets.

Imports logging functionality from myffe.tools.Loggerv1_1 import log_class

__init__ receives important parameter shortcut_nan (missing value processing parameters)
__call__ receives important parameter data (dataset for missing value processing)

Uses shortcut_nan with:
- nan_missing_process_choice (missing value processing method)
  (options: fill_nan_mean, fill_nan_median, fill_nan_mode, fill_nan_forward, fill_nan_backward, fill_nan_intelligent, drop_col_high_missing, fill_nan_knn, skip)
- drop_threshold (missing value processing threshold) (range 0-1, default 0.75)
- knn_n_neighbors (KNN neighbors for missing value processing) (default 5)
"""
import sys
import os

# Add project root directory to Python path
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..', '..')))

from myffe.tools.Loggerv1_1 import log_class
from myffe.tools.color_printer import ColorPrinter
from myffe.tools.evaluation_decorator import with_evaluation
from myffe.preprocessing.function.nanfunction.knn import fill_nan_knn
from myffe.preprocessing.function.nanfunction.intelligent import fill_nan_intelligent
from myffe.preprocessing.function.nanfunction.EI import fill_nan_ei
from myffe.preprocessing.function.nanfunction.drop import drop_high_missing_single_col

@log_class(type='use')
class NanPreprocessing:
    def __init__(self, shortcut_nan=None, label_col=None):            
        self.shortcut_nan = shortcut_nan
        self.label_col = label_col

    @with_evaluation
    def __call__(self, data):
#提前退出
#region 
        import pandas as pd
        if self.shortcut_nan is None:
            ColorPrinter.print_error('错误: 缺少必要的参数 shortcut_nan')
            return data
        if self.shortcut_nan =='skip':
            ColorPrinter.print_error('错误: 缺少必要的参数 shortcut_nan')
            return data
#endregion 
#数据信息展示
#region 
        data_size = data.memory_usage(deep=True).sum() / (1024 * 1024)  # Convert to MB
        ColorPrinter.print_info(f'数据大小: {data_size:.2f} MB')
        ColorPrinter.print_info(f'数据行数: {len(data)} 行')
        self.data = data
        ColorPrinter.print_info('下面进行缺失值处理')
#endregion 
#正式处理
#region 
        processed_data = self.nan_preprocessing_switch(data)
        
        # 只返回特征列，不包含标签列
        return processed_data
#endregion 

    def nan_preprocessing_switch(self, data):        
        #参数获取
        #region 
        import pandas as pd
        nan_ways = self.shortcut_nan.get('nan_ways', [])
        #endregion 

        #参数完整性检查
        #region 
        if not nan_ways:
            ColorPrinter.print_error('错误: 缺少必要的参数 nan_ways')
            return data
        #endregion 

        #创建数据副本
        #region 
        processed_data = data.copy()
        ColorPrinter.print_info('已创建数据副本，开始处理缺失值')
        #endregion 

        #参数展示
        #region 
        ColorPrinter.print_info(f"处理列数: {len(nan_ways)}")
        #endregion 

        #列处理循环
        #region 
        for col_config in nan_ways:
            col = col_config.get('col')
            method = col_config.get('method')
            
            if col in processed_data.columns:
                ColorPrinter.print_info(f"处理列: {col}, 方法: {method}")
                if method == 'skip':         
                    continue
                elif method == 'drop_col_high_missing':
                    # 获取当前列的drop_threshold参数
                    drop_threshold = col_config.get('drop_threshold', 0.75)
                    #构建临时参数字典
                    temp_params = {
                        'drop_threshold': drop_threshold
                    }
                    processed_data = drop_high_missing_single_col(processed_data, col, temp_params)
                elif method == 'fill_nan_knn':
                    if processed_data[col].isna().sum() > 0:
                        # 获取当前列的knn_n_neighbors参数
                        knn_n_neighbors = col_config.get('knn_n_neighbors', 5)
                        # 构建临时参数字典
                        temp_params = {
                            'knn_n_neighbors': knn_n_neighbors
                        }
                        # 直接传递label_col参数，让函数内部处理
                        processed_data = fill_nan_knn(processed_data, target_col=col, shortcut_nan=temp_params, label_col=self.label_col)
                elif method == 'fill_nan_mean':    
                    processed_data[col] = processed_data[col].fillna(processed_data[col].mean())
                    ColorPrinter.print_info(f'fill nan mean: {col}')
                elif method == 'fill_nan_median':    
                    processed_data[col] = processed_data[col].fillna(processed_data[col].median())
                    ColorPrinter.print_info(f'fill nan median: {col}')
                elif method == 'fill_nan_mode':    
                    mode_value = processed_data[col].mode().iloc[0] if not processed_data[col].mode().empty else None
                    if mode_value is not None:
                        processed_data[col] = processed_data[col].fillna(mode_value)
                        ColorPrinter.print_info(f'fill nan mode: {col} = {mode_value}')
                elif method == 'fill_nan_forward':    
                    processed_data[col] = processed_data[col].ffill()
                    ColorPrinter.print_info(f'fill nan forward: {col}')
                elif method == 'fill_nan_backward':    
                    processed_data[col] = processed_data[col].bfill()
                    ColorPrinter.print_info(f'fill nan backward: {col}')
                elif method == 'fill_nan_intelligent':    
                    # 直接传递label_col参数，让函数内部处理
                    processed_data = fill_nan_intelligent(processed_data, target_col=col, label_col=self.label_col)
                    ColorPrinter.print_info(f'fill nan intelligent: {col}')
                elif method == 'fill_nan_ei':    
                    # 获取当前列的模型类型和多项式阶数
                    model_type = col_config.get('ei_model_type', 'linear')
                    poly_degree = col_config.get('poly_degree', 2)
                    
                    # 直接传递label_col参数，让函数内部处理
                    processed_data = fill_nan_ei(processed_data, target_col=col, poly_degree=poly_degree, model_type=model_type, label_col=self.label_col)
                    ColorPrinter.print_info(f'fill nan ei: {col} (model: {model_type}, degree: {poly_degree})')
                elif method == 'specify_fill':    
                    # 获取填充值
                    fill_value = col_config.get('fill_value')
                    if fill_value is not None:
                        processed_data[col] = processed_data[col].fillna(fill_value)
                        ColorPrinter.print_info(f'fill nan with specified value: {col} = {fill_value}')
                    else:
                        ColorPrinter.print_error(f'列 {col} 没有指定填充值')
            else:
                ColorPrinter.print_error(f'指定的列不存在')
                continue
        #endregion 
        return processed_data
    

    







"""
Parameter documentation:

shortcut_nan parameter dictionary:
    - nan_ways: List of column processing configurations
      Type: list of dictionaries
      Default: []
      Description: List of dictionaries, each containing processing configuration for a column
      Each dictionary in nan_ways contains:
        - col: Column name
          Type: string
          Description: Name of the column to process
        - method: Processing method
          Type: string
          Optional values: 'fill_nan_mean', 'fill_nan_median', 'fill_nan_mode', 'fill_nan_forward', 'fill_nan_backward', 'fill_nan_intelligent', 'drop_col_high_missing', 'fill_nan_knn', 'fill_nan_ei', 'specify_fill', 'skip'
          Description: Processing method for the column
        - Additional method-specific parameters:
          - For 'drop_col_high_missing':
            - drop_threshold: Missing value ratio threshold (0.0 - 1.0, default: 0.5)
          - For 'fill_nan_knn':
            - knn_n_neighbors: Number of neighbors (>= 1, default: 5)
          - For 'fill_nan_ei':
            - ei_model_type: Model type ('linear', 'ridge', 'lasso', 'random_forest', 'gradient_boosting', default: 'linear')
            - poly_degree: Polynomial degree (>= 1, default: 2)
          - For 'specify_fill':
            - fill_value: User-specified fill value

Method descriptions:
  - 'fill_nan_mean': Fill missing values with mean
  - 'fill_nan_median': Fill missing values with median
  - 'fill_nan_mode': Fill missing values with mode
  - 'fill_nan_forward': Fill missing values with forward fill
  - 'fill_nan_backward': Fill missing values with backward fill
  - 'fill_nan_intelligent': Fill missing values with intelligent imputation
  - 'drop_col_high_missing': Drop columns with high missing value ratio
  - 'fill_nan_knn': Fill missing values with KNN algorithm
  - 'fill_nan_ei': Fill missing values with ensemble imputation algorithm
  - 'specify_fill': Fill missing values with user-specified values
  - 'skip': Skip missing value processing for this column

Parameter processing logic:
  - Each column's processing configuration is independent and contained in a dictionary within the nan_ways list
  - All possible parameters are included in the configuration as placeholders, regardless of the selected method
  - Unselected parameters are placeheld with default values to ensure consistent configuration structure
  - This design makes the configuration structure more uniform, facilitating subsequent processing and maintenance

Usage example:
    # Apply different missing value processing methods to different columns
    nan_params = {
        'nan_ways': [
            {
                'col': 'numeric_col1',
                'method': 'fill_nan_knn',
                'knn_n_neighbors': 5
            },
            {
                'col': 'numeric_col2',
                'method': 'drop_col_high_missing',
                'drop_threshold': 0.4
            },
            {
                'col': 'categorical_col',
                'method': 'fill_nan_mode'
            },
            {
                'col': 'datetime_col',
                'method': 'fill_nan_forward'
            }
        ]
    }
    nan_processor = NanPreprocessing(shortcut_nan=nan_params, label_col='label')
    cleaned_data = nan_processor(data)
"""


if __name__ == "__main__":
    import pandas as pd
    import numpy as np
    
    ColorPrinter.print_any("=" * 60)
    ColorPrinter.print_info("缺失值处理测试程序")
    ColorPrinter.print_any("=" * 60)
    
    # 创建包含缺失值的测试数据
    np.random.seed(42)
    test_data = pd.DataFrame({
        'feature1': [1, 2, np.nan, 4, 5, np.nan, 7, 8, 9, 10],
        'feature2': [np.nan, 12, 13, np.nan, 15, 16, np.nan, 18, 19, 20],
        'feature3': [21, 22, 23, 24, 25, 26, 27, 28, np.nan, 30],
        'label': ['A', 'B', 'A', 'B', 'A', 'B', 'A', 'B', 'A', 'B']
    })
    
    ColorPrinter.print_any("\n原始数据:")
    ColorPrinter.print_any(test_data)
    ColorPrinter.print_any("\n缺失值统计:")
    ColorPrinter.print_any(test_data.isnull().sum())
    
    # 测试不同的缺失值处理方式
    test_cases = [
        {
            'nan_ways': [
                {'col': 'feature1', 'method': 'fill_nan_mean'},
                {'col': 'feature2', 'method': 'fill_nan_mean'},
                {'col': 'feature3', 'method': 'fill_nan_mean'}
            ]
        },
        {
            'nan_ways': [
                {'col': 'feature1', 'method': 'fill_nan_median'},
                {'col': 'feature2', 'method': 'fill_nan_median'},
                {'col': 'feature3', 'method': 'fill_nan_median'}
            ]
        },
        {
            'nan_ways': [
                {'col': 'feature1', 'method': 'fill_nan_knn', 'knn_n_neighbors': 3},
                {'col': 'feature2', 'method': 'fill_nan_knn', 'knn_n_neighbors': 3},
                {'col': 'feature3', 'method': 'fill_nan_knn', 'knn_n_neighbors': 3}
            ]
        },
        {
            'nan_ways': [
                {'col': 'feature1', 'method': 'fill_nan_forward'},
                {'col': 'feature2', 'method': 'fill_nan_forward'},
                {'col': 'feature3', 'method': 'fill_nan_forward'}
            ]
        },
        {
            'nan_ways': [
                {'col': 'feature1', 'method': 'fill_nan_backward'},
                {'col': 'feature2', 'method': 'fill_nan_backward'},
                {'col': 'feature3', 'method': 'fill_nan_backward'}
            ]
        },
        {
            'nan_ways': [
                {'col': 'feature1', 'method': 'fill_nan_intelligent'},
                {'col': 'feature2', 'method': 'fill_nan_intelligent'},
                {'col': 'feature3', 'method': 'fill_nan_intelligent'}
            ]
        }
    ]
    
    for i, params in enumerate(test_cases, 1):
        ColorPrinter.print_any(f"\n{'=' * 60}")
        ColorPrinter.print_info(f"测试 {i}: {params['nan_ways'][0]['method']}")
        ColorPrinter.print_any('=' * 60)
        
        nan_processor = NanPreprocessing(shortcut_nan=params, label_col='label')
        cleaned_data = nan_processor(test_data.copy())
        
        ColorPrinter.print_any("\n处理后的数据:")
        ColorPrinter.print_any(cleaned_data)
        ColorPrinter.print_any("\n处理后缺失值统计:")
        ColorPrinter.print_any(cleaned_data.isnull().sum())
    
    # 全面渗透测试套件
    ColorPrinter.print_any(f"\n{'=' * 80}")
    ColorPrinter.print_info("全面渗透测试套件")
    ColorPrinter.print_any('=' * 80)
    
    # 测试1: 列级参数测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试1: 列级参数测试")
    ColorPrinter.print_any('-' * 60)
    
    test_data_col_level = pd.DataFrame({
        'col1': [1, 2, np.nan, 4, 5, 6, 7, 8, 9, 10],
        'col2': [11, 12, 13, np.nan, 15, 16, 17, 18, 19, 20],
        'col3': [21, 22, 23, 24, 25, 26, 27, 28, 29, 30],
        'label': ['A', 'B', 'A', 'B', 'A', 'B', 'A', 'B', 'A', 'B']
    })
    
    params_col_level = {
        'nan_ways': [
            {'col': 'col1', 'method': 'fill_nan_mean'},
            {'col': 'col2', 'method': 'fill_nan_median'},
            {'col': 'col3', 'method': 'fill_nan_mode'}
        ]
    }
    
    nan_processor_col_level = NanPreprocessing(shortcut_nan=params_col_level, label_col='label')
    result_col_level = nan_processor_col_level(test_data_col_level.copy())
    
    ColorPrinter.print_any("\n处理结果:")
    ColorPrinter.print_any(result_col_level)
    ColorPrinter.print_any(f"\n数据形状变化: {test_data_col_level.shape} -> {result_col_level.shape}")
    ColorPrinter.print_info("期望：col1均值填充，col2中位数填充，col3众数填充")
    
    # 测试2: 前向和后向填充测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试2: 前向和后向填充测试")
    ColorPrinter.print_any('-' * 60)
    
    test_data_forward_backward = pd.DataFrame({
        'col1': [1, np.nan, np.nan, 4, 5, 6, 7, 8, 9, 10],
        'col2': [11, 12, 13, 14, np.nan, np.nan, 17, 18, 19, 20],
        'label': ['A', 'B', 'A', 'B', 'A', 'B', 'A', 'B', 'A', 'B']
    })
    
    params_forward_backward = {
        'nan_ways': [
            {'col': 'col1', 'method': 'fill_nan_forward'},
            {'col': 'col2', 'method': 'fill_nan_backward'}
        ]
    }
    
    nan_processor_forward_backward = NanPreprocessing(shortcut_nan=params_forward_backward, label_col='label')
    result_forward_backward = nan_processor_forward_backward(test_data_forward_backward.copy())
    
    ColorPrinter.print_any("\n处理结果:")
    ColorPrinter.print_any(result_forward_backward)
    ColorPrinter.print_info("期望：col1前向填充，col2后向填充")
    
    # 测试3: 智能填充测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试3: 智能填充测试")
    ColorPrinter.print_any('-' * 60)
    
    test_data_intelligent = pd.DataFrame({
        'col1': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        'col2': [1.1, 2.2, 3.3, 4.4, 5.5, 6.6, 7.7, 8.8, 9.9, 10.1],
        'col3': [1, 2, np.nan, 4, 5, 6, 7, 8, 9, 10],
        'label': ['A', 'B', 'A', 'B', 'A', 'B', 'A', 'B', 'A', 'B']
    })
    
    params_intelligent = {
        'nan_ways': [
            {'col': 'col3', 'method': 'fill_nan_intelligent'}
        ]
    }
    
    nan_processor_intelligent = NanPreprocessing(shortcut_nan=params_intelligent, label_col='label')
    result_intelligent = nan_processor_intelligent(test_data_intelligent.copy())
    
    ColorPrinter.print_any("\n处理结果:")
    ColorPrinter.print_any(result_intelligent)
    ColorPrinter.print_info("期望：col3使用col1或col2进行智能填充")
    
    # 测试4: 删除高缺失值列测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试4: 删除高缺失值列测试")
    ColorPrinter.print_any('-' * 60)
    
    test_data_drop_col = pd.DataFrame({
        'col1': [1, 2, np.nan, 4, 5, 6, 7, 8, 9, 10],
        'col2': [11, 12, np.nan, 14, 15, np.nan, 17, 18, np.nan, 20],
        'col3': [21, 22, 23, 24, 25, 26, 27, 28, 29, 30],
        'label': ['A', 'B', 'A', 'B', 'A', 'B', 'A', 'B', 'A', 'B']
    })
    
    params_drop_col = {
        'nan_ways': [
            {'col': 'col1', 'method': 'drop_col_high_missing', 'drop_threshold': 0.3},
            {'col': 'col2', 'method': 'drop_col_high_missing', 'drop_threshold': 0.3},
            {'col': 'col3', 'method': 'drop_col_high_missing', 'drop_threshold': 0.3}
        ]
    }
    
    nan_processor_drop_col = NanPreprocessing(shortcut_nan=params_drop_col, label_col='label')
    result_drop_col = nan_processor_drop_col(test_data_drop_col.copy())
    
    ColorPrinter.print_any("\n处理结果:")
    ColorPrinter.print_any(result_drop_col)
    ColorPrinter.print_any(f"\n数据形状变化: {test_data_drop_col.shape} -> {result_drop_col.shape}")
    ColorPrinter.print_info("期望：删除col2（缺失值比例33.3% > 30%阈值）")
    
    # 测试5: KNN填充测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试5: KNN填充测试")
    ColorPrinter.print_any('-' * 60)
    
    test_data_knn = pd.DataFrame({
        'col1': [1, 2, np.nan, 4, 5, 6, 7, 8, 9, 10],
        'col2': [11, 12, 13, np.nan, 15, 16, 17, 18, 19, 20],
        'col3': [21, 22, 23, 24, 25, 26, 27, 28, 29, 30],
        'label': ['A', 'B', 'A', 'B', 'A', 'B', 'A', 'B', 'A', 'B']
    })
    
    params_knn = {
        'nan_ways': [
            {'col': 'col1', 'method': 'fill_nan_knn', 'knn_n_neighbors': 3},
            {'col': 'col2', 'method': 'fill_nan_knn', 'knn_n_neighbors': 3}
        ]
    }
    
    nan_processor_knn = NanPreprocessing(shortcut_nan=params_knn, label_col='label')
    result_knn = nan_processor_knn(test_data_knn.copy())
    
    ColorPrinter.print_any("\n处理结果:")
    ColorPrinter.print_any(result_knn)
    ColorPrinter.print_info("期望：col1和col2使用KNN填充缺失值")
    
    # 测试6: 组合处理测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试6: 组合处理测试")
    ColorPrinter.print_any('-' * 60)
    
    test_data_combined = pd.DataFrame({
        'col1': [1, 2, np.nan, 4, 5, 6, 7, 8, 9, 10],
        'col2': [11, 12, 13, np.nan, 15, 16, 17, 18, 19, 20],
        'col3': [21, 22, 23, 24, 25, 26, 27, 28, 29, 30],
        'label': ['A', 'B', 'A', 'B', 'A', 'B', 'A', 'B', 'A', 'B']
    })
    
    params_combined = {
        'nan_ways': [
            {'col': 'col1', 'method': 'fill_nan_mean'},
            {'col': 'col2', 'method': 'fill_nan_median'},
            {'col': 'col3', 'method': 'fill_nan_mode'}
        ]
    }
    
    nan_processor_combined = NanPreprocessing(shortcut_nan=params_combined, label_col='label')
    result_combined = nan_processor_combined(test_data_combined.copy())
    
    ColorPrinter.print_any("\n处理结果:")
    ColorPrinter.print_any(result_combined)
    ColorPrinter.print_info("期望：col1均值填充，col2中位数填充，col3众数填充")
    
    # 测试7: 边界情况测试 - 无缺失值
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试7: 边界情况测试 - 无缺失值")
    ColorPrinter.print_any('-' * 60)
    
    test_data_no_missing = pd.DataFrame({
        'col1': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        'col2': [11, 12, 13, 14, 15, 16, 17, 18, 19, 20],
        'label': ['A', 'B', 'A', 'B', 'A', 'B', 'A', 'B', 'A', 'B']
    })
    
    params_no_missing = {
        'nan_ways': [
            {'col': 'col1', 'method': 'fill_nan_mean'},
            {'col': 'col2', 'method': 'fill_nan_mean'}
        ]
    }
    
    nan_processor_no_missing = NanPreprocessing(shortcut_nan=params_no_missing, label_col='label')
    result_no_missing = nan_processor_no_missing(test_data_no_missing.copy())
    
    ColorPrinter.print_any("\n处理结果:")
    ColorPrinter.print_any(result_no_missing)
    ColorPrinter.print_info("期望：数据保持不变，无缺失值处理")
    
    # 测试8: 边界情况测试 - 全部缺失值
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试8: 边界情况测试 - 全部缺失值")
    ColorPrinter.print_any('-' * 60)
    
    test_data_all_missing = pd.DataFrame({
        'col1': [np.nan, np.nan, np.nan, np.nan, np.nan],
        'col2': [np.nan, np.nan, np.nan, np.nan, np.nan],
        'label': ['A', 'B', 'A', 'B', 'A']
    })
    
    params_all_missing = {
        'nan_ways': [
            {'col': 'col1', 'method': 'fill_nan_mean'},
            {'col': 'col2', 'method': 'fill_nan_mean'}
        ]
    }
    
    nan_processor_all_missing = NanPreprocessing(shortcut_nan=params_all_missing, label_col='label')
    result_all_missing = nan_processor_all_missing(test_data_all_missing.copy())
    
    ColorPrinter.print_any("\n处理结果:")
    ColorPrinter.print_any(result_all_missing)
    ColorPrinter.print_info("期望：均值填充，但可能无法计算均值")
    
    # 测试9: 异常情况测试 - 空数据
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试9: 异常情况测试 - 空数据")
    ColorPrinter.print_any('-' * 60)
    
    test_data_empty = pd.DataFrame({
        'col1': [],
        'col2': [],
        'label': []
    })
    
    params_empty = {
        'nan_ways': [
            {'col': 'col1', 'method': 'fill_nan_mean'},
            {'col': 'col2', 'method': 'fill_nan_mean'}
        ]
    }
    
    try:
        nan_processor_empty = NanPreprocessing(shortcut_nan=params_empty, label_col='label')
        result_empty = nan_processor_empty(test_data_empty.copy())
        
        ColorPrinter.print_any("\n处理结果:")
        ColorPrinter.print_any(result_empty)
        ColorPrinter.print_info("期望：空数据保持不变")
    except Exception as e:
        ColorPrinter.print_error(f"异常处理: {e}")
        ColorPrinter.print_info("期望：空数据应该能正常处理")
    
    # 测试10: 不同数据类型测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试10: 不同数据类型测试")
    ColorPrinter.print_any('-' * 60)
    
    test_data_types = pd.DataFrame({
        'numeric_col': [1, 2, np.nan, 4, 5],
        'string_col': ['A', 'B', np.nan, 'D', 'E'],
        'float_col': [1.1, 2.2, np.nan, 4.4, 5.5],
        'int_col': [10, 20, np.nan, 40, 50],
        'label': ['A', 'B', 'A', 'B', 'A']
    })
    
    params_types = {
        'nan_ways': [
            {'col': 'numeric_col', 'method': 'fill_nan_mean'},
            {'col': 'string_col', 'method': 'fill_nan_mode'},
            {'col': 'float_col', 'method': 'fill_nan_mean'},
            {'col': 'int_col', 'method': 'fill_nan_median'}
        ]
    }
    
    nan_processor_types = NanPreprocessing(shortcut_nan=params_types, label_col='label')
    result_types = nan_processor_types(test_data_types.copy())
    
    ColorPrinter.print_any("\n处理结果:")
    ColorPrinter.print_any(result_types)
    ColorPrinter.print_info("期望：数值列均值填充，字符串列众数填充")
    
    # 测试11: 大数据集测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试11: 大数据集测试")
    ColorPrinter.print_any('-' * 60)
    
    np.random.seed(42)
    large_data_size = 10000
    test_data_large = pd.DataFrame({
        'feature1': np.random.randn(large_data_size),
        'feature2': np.random.randn(large_data_size),
        'feature3': np.random.randn(large_data_size),
        'label': np.random.choice(['A', 'B'], large_data_size)
    })
    
    # 随机添加缺失值
    missing_indices1 = np.random.choice(large_data_size, int(large_data_size * 0.1), replace=False)
    missing_indices2 = np.random.choice(large_data_size, int(large_data_size * 0.15), replace=False)
    test_data_large.loc[missing_indices1, 'feature1'] = np.nan
    test_data_large.loc[missing_indices2, 'feature2'] = np.nan
    
    params_large = {
        'nan_ways': [
            {'col': 'feature1', 'method': 'fill_nan_mean'},
            {'col': 'feature2', 'method': 'fill_nan_mean'},
            {'col': 'feature3', 'method': 'fill_nan_mean'}
        ]
    }
    
    nan_processor_large = NanPreprocessing(shortcut_nan=params_large, label_col='label')
    result_large = nan_processor_large(test_data_large.copy())
    
    ColorPrinter.print_any(f"\n处理结果（数据大小: {large_data_size} 行）:")
    ColorPrinter.print_any(f"原始缺失值: {test_data_large.isnull().sum().sum()}")
    ColorPrinter.print_any(f"处理后缺失值: {result_large.isnull().sum().sum()}")
    ColorPrinter.print_info("期望：所有缺失值被填充")
    
    # 测试12: 极端阈值测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试12: 极端阈值测试")
    ColorPrinter.print_any('-' * 60)
    
    test_data_threshold = pd.DataFrame({
        'col1': [1, 2, np.nan, 4, 5, 6, 7, 8, 9, 10],
        'col2': [11, 12, np.nan, 14, 15, np.nan, 17, 18, np.nan, 20],
        'col3': [21, 22, 23, 24, 25, 26, 27, 28, 29, 30],
        'label': ['A', 'B', 'A', 'B', 'A', 'B', 'A', 'B', 'A', 'B']
    })
    
    params_threshold_low = {
        'nan_ways': [
            {'col': 'col1', 'method': 'drop_col_high_missing', 'drop_threshold': 0.1},
            {'col': 'col2', 'method': 'drop_col_high_missing', 'drop_threshold': 0.1},
            {'col': 'col3', 'method': 'drop_col_high_missing', 'drop_threshold': 0.1}
        ]
    }
    
    params_threshold_high = {
        'nan_ways': [
            {'col': 'col1', 'method': 'drop_col_high_missing', 'drop_threshold': 0.9},
            {'col': 'col2', 'method': 'drop_col_high_missing', 'drop_threshold': 0.9},
            {'col': 'col3', 'method': 'drop_col_high_missing', 'drop_threshold': 0.9}
        ]
    }
    
    nan_processor_threshold_low = NanPreprocessing(shortcut_nan=params_threshold_low, label_col='label')
    result_threshold_low = nan_processor_threshold_low(test_data_threshold.copy())
    
    nan_processor_threshold_high = NanPreprocessing(shortcut_nan=params_threshold_high, label_col='label')
    result_threshold_high = nan_processor_threshold_high(test_data_threshold.copy())
    
    ColorPrinter.print_any("\n低阈值（0.1）结果:")
    ColorPrinter.print_any(f"删除列数: {test_data_threshold.shape[1] - result_threshold_low.shape[1]}")
    ColorPrinter.print_any("高阈值（0.9）结果:")
    ColorPrinter.print_any(f"删除列数: {test_data_threshold.shape[1] - result_threshold_high.shape[1]}")
    
    # 测试13: 标签列保护测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试13: 标签列保护测试")
    ColorPrinter.print_any('-' * 60)
    
    test_data_label = pd.DataFrame({
        'feature1': [1, 2, np.nan, 4, 5],
        'feature2': [11, 12, np.nan, 14, 15],
        'label': [np.nan, 'B', np.nan, 'B', 'A']  # 标签列也有缺失值
    })
    
    params_label = {
        'nan_ways': [
            {'col': 'feature1', 'method': 'fill_nan_mean'},
            {'col': 'feature2', 'method': 'fill_nan_mean'}
        ]
    }
    
    nan_processor_label = NanPreprocessing(shortcut_nan=params_label, label_col='label')
    result_label = nan_processor_label(test_data_label.copy())
    
    ColorPrinter.print_any("\n处理结果:")
    ColorPrinter.print_any(result_label)
    ColorPrinter.print_info("期望：标签列的缺失值不被处理")
    
    # 测试14: 多列同时处理测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试14: 多列同时处理测试")
    ColorPrinter.print_any('-' * 60)
    
    test_data_multi = pd.DataFrame({
        'col1': [1, 2, np.nan, 4, 5, 6, 7, 8, 9, 10],
        'col2': [11, 12, 13, np.nan, 15, 16, 17, 18, 19, 20],
        'col3': [21, 22, 23, 24, 25, 26, 27, 28, 29, 30],
        'col4': [31, 32, 33, 34, 35, 36, 37, 38, 39, 40],
        'col5': [41, 42, 43, 44, 45, 46, 47, 48, 49, 50],
        'label': ['A', 'B', 'A', 'B', 'A', 'B', 'A', 'B', 'A', 'B']
    })
    
    params_multi = {
        'nan_ways': [
            {'col': 'col1', 'method': 'fill_nan_mean'},
            {'col': 'col2', 'method': 'fill_nan_median'},
            {'col': 'col3', 'method': 'fill_nan_mode'},
            {'col': 'col4', 'method': 'fill_nan_forward'},
            {'col': 'col5', 'method': 'fill_nan_backward'}
        ]
    }
    
    nan_processor_multi = NanPreprocessing(shortcut_nan=params_multi, label_col='label')
    result_multi = nan_processor_multi(test_data_multi.copy())
    
    ColorPrinter.print_any("\n处理结果:")
    ColorPrinter.print_any(f"数据形状: {test_data_multi.shape} -> {result_multi.shape}")
    ColorPrinter.print_info("期望：每列使用不同的填充方法")
    
    # 测试15: 性能测试
    ColorPrinter.print_any(f"\n{'-' * 60}")
    ColorPrinter.print_info("测试15: 性能测试")
    ColorPrinter.print_any('-' * 60)
    
    import time
    test_data_perf = pd.DataFrame({
        'feature1': np.random.randn(5000),
        'feature2': np.random.randn(5000),
        'feature3': np.random.randn(5000),
        'label': np.random.choice(['A', 'B'], 5000)
    })
    
    # 随机添加缺失值
    for col in ['feature1', 'feature2', 'feature3']:
        missing_indices = np.random.choice(5000, int(5000 * 0.1), replace=False)
        test_data_perf.loc[missing_indices, col] = np.nan
    
    params_perf = {
        'nan_ways': [
            {'col': 'feature1', 'method': 'fill_nan_mean'},
            {'col': 'feature2', 'method': 'fill_nan_mean'},
            {'col': 'feature3', 'method': 'fill_nan_mean'}
        ]
    }
    
    nan_processor_perf = NanPreprocessing(shortcut_nan=params_perf, label_col='label')
    
    start_time = time.time()
    result_perf = nan_processor_perf(test_data_perf.copy())
    end_time = time.time()
    
    ColorPrinter.print_any(f"\n处理时间: {end_time - start_time:.4f} 秒")
    ColorPrinter.print_any(f"数据大小: {len(test_data_perf)} 行")
    ColorPrinter.print_any(f"原始缺失值: {test_data_perf.isnull().sum().sum()}")
    ColorPrinter.print_any(f"处理后缺失值: {result_perf.isnull().sum().sum()}")
    
    ColorPrinter.print_any("\n" + "=" * 80)
    ColorPrinter.print_success("全面渗透测试完成！")
    ColorPrinter.print_any("=" * 80)



import numpy as np

def ok_input(input_type, max_value=np.inf, min_value=-np.inf):
    """
    交互式输入验证
    
    提供类型安全的交互式输入功能，支持整数、浮点数和字符串输入，
    自动进行范围验证和错误处理。
    
    Args:
        input_type (str): 输入类型，支持'int'、'float'或'str'
        max_value (float): 最大值限制，仅适用于数字类型，默认为正无穷
        min_value (float): 最小值限制，仅适用于数字类型，默认为负无穷
        
    Returns:
        int, float or str: 验证通过的用户输入值
        
    Raises:
        ValueError: 当输入类型不匹配或超出范围时
        
    Example:
        # 输入整数
        n_estimators = ok_input('int', min_value=1, max_value=1000)
        
        # 输入浮点数
        learning_rate = ok_input('float', min_value=0.01, max_value=1.0)
        
        # 输入字符串
        name = ok_input('str')
    """
    # 对于字符串类型，使用普通的input函数
    if input_type == 'str':
        return input('请输入字符串：')
    
    # 对于数字类型，进行类型转换和范围验证
    while True:
        try:
            if input_type == 'int':
                value = int(input(f'请输入{input_type}({min_value}-{max_value}之间的整数)：'))
                if value <= min_value or value >= max_value:
                    print(f'输入错误请输入{min_value}-{max_value}之间的整数')
                    continue
            elif input_type == 'float':
                value = float(input(f'请输入{input_type}({min_value}-{max_value}之间的小数)：'))
                if value <= min_value or value >= max_value:
                    print(f'输入错误请输入{min_value}-{max_value}之间的小数')
                    continue
            else:
                print('输入错误请输入int、float或str')
                continue
            return value
        except ValueError:
            print(f'输入错误请输入有效的{input_type}')
            continue